"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var _ = require("underscore");
var CONFIG_DEFAULTS = {
    filterPattern: '',
    normalizedFilterID: true,
    createLambdaPermission: true,
};
var LogForwardingPlugin = /** @class */ (function () {
    function LogForwardingPlugin(serverless, options) {
        this.config = null;
        // The key of a lambda permission in CloudFormation resources
        this.permissionId = 'LogForwardingLambdaPermission';
        this.serverless = serverless;
        this.options = options;
        this.provider = this.serverless.getProvider('aws');
        this.hooks = {
            'package:initialize': this.updateResources.bind(this),
        };
    }
    LogForwardingPlugin.prototype.loadConfig = function () {
        var service = this.serverless.service;
        if (!service.custom || !service.custom.logForwarding) {
            throw new Error('Serverless-log-forwarding configuration not provided.');
        }
        this.config = __assign(__assign({}, CONFIG_DEFAULTS), service.custom.logForwarding);
        if (this.config.destinationARN === undefined) {
            throw new Error('Serverless-log-forwarding is not configured correctly. Please see README for proper setup.');
        }
    };
    LogForwardingPlugin.prototype.updateResources = function () {
        this.loadConfig();
        var stage = this.getStage();
        if (this.config.stages && !this.config.stages.includes(stage)) {
            this.serverless.cli.log("Log Forwarding is ignored for ".concat(stage, " stage"));
            return;
        }
        this.serverless.cli.log('Updating Log Forwarding Resources...');
        var resourceObj = this.createResourcesObj();
        var cfResources = this.getResources();
        _.extend(cfResources, resourceObj);
        this.serverless.cli.log('Log Forwarding Resources Updated');
    };
    LogForwardingPlugin.prototype.getResources = function () {
        var service = this.serverless.service;
        if (service.resources === undefined) {
            service.resources = {
                Resources: {},
            };
        }
        if (service.resources.Resources === undefined) {
            service.resources.Resources = {};
        }
        return service.resources.Resources;
    };
    LogForwardingPlugin.prototype.createResourcesObj = function () {
        var _this = this;
        var service = this.serverless.service;
        var resourceObj = {};
        var createLambdaPermission = this.config.createLambdaPermission && !this.config.roleArn;
        if (createLambdaPermission) {
            var permission = this.makeLambdaPermission();
            resourceObj[this.permissionId] = permission;
        }
        _.keys(service.functions)
            .filter(function (functionName) {
            var _a = _this.serverless.service.getFunction(functionName).logForwarding, logForwarding = _a === void 0 ? {} : _a;
            if (logForwarding.enabled === undefined) {
                return true;
            }
            return !!logForwarding.enabled;
        })
            .forEach(function (functionName) {
            var filterId = _this.getFilterId(functionName);
            var dependsOn = createLambdaPermission ? [_this.permissionId] : [];
            var filter = _this.makeSubsctiptionFilter(functionName, dependsOn);
            resourceObj[filterId] = filter;
        });
        return resourceObj;
    };
    LogForwardingPlugin.prototype.getFilterId = function (functionName) {
        var filterName = this.config.normalizedFilterID
            ? this.provider.naming.getNormalizedFunctionName(functionName)
            : functionName;
        return "SubscriptionFilter".concat(filterName);
    };
    LogForwardingPlugin.prototype.getStage = function () {
        var stage = this.options.stage;
        if (stage && stage !== '') {
            return stage;
        }
        return this.serverless.service.provider.stage;
    };
    LogForwardingPlugin.prototype.makeSubsctiptionFilter = function (functionName, deps) {
        var functionObject = this.serverless.service.getFunction(functionName);
        var logGroupName = this.provider.naming.getLogGroupName(functionObject.name);
        var logGroupId = this.provider.naming.getLogGroupLogicalId(functionName);
        var roleObject = this.config.roleArn ? { RoleArn: this.config.roleArn } : {};
        return {
            Type: 'AWS::Logs::SubscriptionFilter',
            Properties: __assign({ DestinationArn: this.config.destinationARN, FilterPattern: this.config.filterPattern, LogGroupName: logGroupName }, roleObject),
            DependsOn: _.union(deps, [logGroupId]),
        };
    };
    LogForwardingPlugin.prototype.makeLambdaPermission = function () {
        var region = this.serverless.service.provider.region;
        var principal = "logs.".concat(region, ".amazonaws.com");
        return {
            Type: 'AWS::Lambda::Permission',
            Properties: {
                FunctionName: this.config.destinationARN,
                Action: 'lambda:InvokeFunction',
                Principal: principal,
            },
        };
    };
    return LogForwardingPlugin;
}());
module.exports = LogForwardingPlugin;
